from .add_center import add_center
from .add_area import add_area
from .add_circularity import add_circularity
from .scale_json import scale_json
from .translation import translation_json
from .cropping import crop_json
from .flipping import flip_json
from .rotation import rotate_json
from .summary_json import summary_json


# -*- coding: utf-8 -*-
"""Tomni is a collection of image analysis functions useful for CytoSmart solution."""

__author__ = """Tom Nijhof & Jelle van Kerkvoorde"""
__email__ = "tom.nijhof@cytosmart.com"
__version__ = "1.11.0"

from .bbox_fitting import bbox_fitting, bbox_fitting_center
from .img_paste import img_paste
from .shape_fitting import fit_rect_around_ellipse
from .img_dim import img_dim

from . import illumination_correction
from . import make_mask
from . import transformers
from . import json_operations
from . import bbox_operations



from __future__ import absolute_import
from unittest import TestCase
from .main import fit_rect_around_ellipse
import numpy as np

class Testfit_rect_around_ellipse(TestCase):
    def test_fit_circle(self):
        x = 0
        y = 100
        r1 = 900
        r2 = 900
        alpha = 0

        expectedX1 = -900
        expectedY1 = -800
        expectedX2 = 900
        expectedY2 = 1000

        x1, y1, x2, y2 = fit_rect_around_ellipse(x, y, r1, r2, alpha)

        self.assertEqual(x1, expectedX1)
        self.assertEqual(y1, expectedY1)
        self.assertEqual(x2, expectedX2)
        self.assertEqual(y2, expectedY2)

    def test_fit_ellipse_no_rotation(self):
        x = 0
        y = 100
        r1 = 900
        r2 = 500
        alpha = 0

        expectedX1 = -900
        expectedY1 = -400
        expectedX2 = 900
        expectedY2 = 600

        x1, y1, x2, y2 = fit_rect_around_ellipse(x, y, r1, r2, alpha)

        self.assertEqual(x1, expectedX1)
        self.assertEqual(y1, expectedY1)
        self.assertEqual(x2, expectedX2)
        self.assertEqual(y2, expectedY2)


    def test_fit_ellipse_with_rotation(self):
        '''
        These number where measured using InkScape 
        '''
        x = 108
        y = 185
        r1 = 79
        r2 = 50
        alpha = 30

        expectedX1 = 35
        expectedY1 = 126
        expectedX2 = 181
        expectedY2 = 244

        x1, y1, x2, y2 = fit_rect_around_ellipse(x, y, r1, r2, alpha)

        print(x1, y1, x2, y2)
        print("width {} expected width {}".format(x2 - x1, expectedX2 - expectedX1))
        print("height {} expected height {}".format(y2 - y1, expectedY2 - expectedY1))
        self.assertAlmostEqual(x1, expectedX1, 0)
        self.assertAlmostEqual(y1, expectedY1, 0)
        self.assertAlmostEqual(x2, expectedX2, 0)
        self.assertAlmostEqual(y2, expectedY2, 0)


from __future__ import absolute_import
from unittest import TestCase
from .main import json2dict


class TestJson2Dict(TestCase):
    def test_default(self):
        input_json_list = [
            {
                "type": "polygon",
                "accuracy": 0.45455,
                "parents": [],
                "children": [],
                "id": "unicorn",
                "points": [
                    {"x": 505, "y": 745},
                    {"x": 478, "y": 756},
                ],
                "area": 300,
            },
            {
                "type": "ellipse",
                "accuracy": 0.443,
                "id": "unicorn",
                "center": {"x": 769, "y": 649},
                "radiusX": 24,
                "radiusY": 23,
                "angleOfRotation": 0,
                "area": 3,
            },
        ]

        expected_output = {
            "type": ["polygon", "ellipse"],
            "area": [300, 3],
            "id": ["unicorn", "unicorn"],
            "center": [None, [769, 649]],
            "index": [0, 1],
        }

        result = json2dict(input_json_list)

        self.assertDictEqual(result, expected_output)

    def test_singel_keyword(self):
        input_json_list = [
            {
                "type": "polygon",
                "accuracy": 0.45455,
                "parents": [],
                "children": [],
                "id": "unicorn",
                "points": [
                    {"x": 505, "y": 745},
                    {"x": 478, "y": 756},
                ],
                "area": 300,
            },
            {
                "type": "ellipse",
                "accuracy": 0.443,
                "id": "unicorn",
                "center": {"x": 769, "y": 649},
                "radiusX": 24,
                "radiusY": 23,
                "angleOfRotation": 0,
                "area": 3,
            },
        ]

        expected_output = {
            "radiusX": [None, 24],
            "index": [0, 1],
        }

        result = json2dict(input_json_list, ["radiusX"])

        self.assertDictEqual(result, expected_output)

    def test_multi_keyword(self):
        input_json_list = [
            {
                "type": "polygon",
                "accuracy": 0.45455,
                "parents": [],
                "children": [],
                "id": "unicorn",
                "points": [
                    {"x": 505, "y": 745},
                    {"x": 478, "y": 756},
                ],
                "area": 300,
            },
            {
                "type": "ellipse",
                "accuracy": 0.443,
                "id": "unicorn",
                "center": {"x": 769, "y": 649},
                "radiusX": 24,
                "radiusY": 23,
                "angleOfRotation": 0,
                "area": 3,
            },
        ]

        expected_output = {
            "points": [
                [
                    {"x": 505, "y": 745},
                    {"x": 478, "y": 756},
                ],
                None,
            ],
            "radiusX": [None, 24],
            "index": [0, 1],
        }

        result = json2dict(input_json_list, ["radiusX", "points"])

        self.assertDictEqual(result, expected_output)

    def test_unknown_keyword(self):
        input_json_list = [
            {
                "type": "polygon",
                "accuracy": 0.45455,
                "parents": [],
                "children": [],
                "id": "unicorn",
                "points": [
                    {"x": 505, "y": 745},
                    {"x": 478, "y": 756},
                ],
                "area": 300,
            },
            {
                "type": "ellipse",
                "accuracy": 0.443,
                "id": "unicorn",
                "center": {"x": 769, "y": 649},
                "radiusX": 24,
                "radiusY": 23,
                "angleOfRotation": 0,
                "area": 3,
            },
        ]

        expected_output = {
            "cytosmart": [None, None],
            "index": [0, 1],
        }

        result = json2dict(input_json_list, ["cytosmart"])

        self.assertDictEqual(result, expected_output)


import numpy as np


def mask2bbox(mask: np.ndarray, padding: int = 0) -> tuple:
    """
    Convert a binary image mask into bounding box coordinates (xmin, ymin, xmax, ymax).

    Args:
        mask (np.ndarray): a binary image mask.
        padding (int, optional): Padding add/subtracted from the bounding box
            (xmin-padding, ymin-padding, xmax+padding, ymax+padding). The bounding box coordinates
            do not get outside the image dimensions. e.g bbox (1,3,1,3) image_dim (2,2), padding 2;
            resulting bbox (0, 3, 0,3). Defaults to 0.

    Returns:
        tuple: (xmin, ymin, xmax, ymax)
    """
    x1, x2, y1, y2 = (0, 0, 0, 0)

    max_x = np.max(mask, axis=0)
    max_y = np.max(mask, axis=1)

    x_nonzeros = np.nonzero(max_x)
    y_nonzeros = np.nonzero(max_y)

    if x_nonzeros:
        if len(x_nonzeros[0] > 0):
            x1 = np.nonzero(max_x)[0][0]
            x2 = np.nonzero(max_x)[0][-1] + 1
    else:
        return (x1, y1, x2, y2)

    if y_nonzeros:
        if len(y_nonzeros[0]) > 0:
            y1 = np.nonzero(max_y)[0][0]
            y2 = np.nonzero(max_y)[0][-1] + 1
    else:
        return (x1, y1, x2, y2)

    if x1 == y1 == x2 == y2 == 0:
        return (x1, y1, x2, y2)

    # Add padding to bounding box
    if x1 - padding < 0:
        x1 = 0
    else:
        x1 -= padding

    if x2 + padding >= mask.shape[1]:
        x2 = mask.shape[1]
    else:
        x2 += padding

    if y1 - padding < 0:
        y1 = 0
    else:
        y1 -= padding

    if y2 + padding >= mask.shape[0]:
        y2 = mask.shape[0]
    else:
        y2 += padding

    return (x1, y1, x2, y2)



def scale_json(json_object: dict, scaling_factor: float):
    """
    Takes in a json object (ellipse or polygon) and makes it scaling_factor bigger
    json_object: (dict) following the standard format
    scaling_factor: (float) the multiplier of the object

    example:
    json_object = {
        "type": "ellipse", 
        "center": {"x": 100, "y": 200}, 
        "radiusX": 50, 
        "radiusY": 100, 
        "angleOfRotation": 0, 
        "id": "12345-abcde"}
    scaling_factor = 2

    result = {
        "type": "ellipse", 
        "center": {"x": 200, "y": 400}, 
        "radiusX": 100, 
        "radiusY": 200, 
        "angleOfRotation": 0, 
        "id": "12345-abcde"}

    """
    newjson_object = json_object.copy()
    if json_object["type"] == "ellipse":
        newjson_object["radiusX"] = int(round(json_object["radiusX"] * scaling_factor))
        newjson_object["radiusY"] = int(round(json_object["radiusY"] * scaling_factor))
        newjson_object["center"]["x"] = int(
            round(json_object["center"]["x"] * scaling_factor)
        )
        newjson_object["center"]["y"] = int(
            round(json_object["center"]["y"] * scaling_factor)
        )

    elif json_object["type"] == "polygon":
        all_points = []
        for point in json_object["points"]:
            all_points.append(
                {
                    "x": int(round(point["x"] * scaling_factor)),
                    "y": int(round(point["y"] * scaling_factor)),
                }
            )
        newjson_object["points"] = all_points
    else:
        raise ValueError(f"type {json_object['type']} is not supported")

    return newjson_object




import cv2
import numpy as np


def contour2bbox(contour: np.ndarray) -> tuple:
    """
    Converts opencv2 contours into (xmin, ymin, xmax, ymax) bounding box format

    Args:
        contour (np.ndarray): an array with the coordinates that defines the contour of a object
        e.g
        Contour
        array( [[[3, 3]],

                [[3, 5]],

                [[5, 5]],

                [[5, 3]]], dtype=int32)

        The object
        array( [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                [0, 0, 0, 1, 1, 1, 0, 0, 0, 0],
                [0, 0, 0, 1, 1, 1, 0, 0, 0, 0],
                [0, 0, 0, 1, 1, 1, 0, 0, 0, 0],
                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]], dtype=uint8)

        boundingbox (3, 3, 6, 6)

    Returns:
        tuple: (xmin, ymin, xmax, ymax)
    """
    x_min, y_min, w, h = cv2.boundingRect(contour)
    return (x_min, y_min, x_min + w, y_min + h)



from unittest import TestCase
import numpy as np
from .main import list_of_points2json

class TestContour2json(TestCase):
    def test_happy_flow_np_array(self):
        list_of_points = np.array([[500, 50], [500, 99], [539, 99], [539, 50]])

        expectedResult = {
            "type": "polygon",
            "points": [
                {"x": 500, "y": 50},
                {"x": 500, "y": 99},
                {"x": 539, "y": 99},
                {"x": 539, "y": 50},
            ],
            "id": "unicorn",
            "parents": [],
            "children": [],
        }

        result = list_of_points2json(list_of_points)
        result["id"] = "unicorn"

        self.assertDictEqual(result, expectedResult)

    def test_happy_flow_list(self):
        list_of_points = [[600, 50], [600, 99], [412, 99], [412, 50]]

        expectedResult = {
            "type": "polygon",
            "points": [
                {"x": 600, "y": 50},
                {"x": 600, "y": 99},
                {"x": 412, "y": 99},
                {"x": 412, "y": 50},
            ],
            "id": "unicorn",
            "parents": [],
            "children": [],
        }

        result = list_of_points2json(list_of_points)
        result["id"] = "unicorn"

        self.assertDictEqual(result, expectedResult)


import cv2
import numpy as np

from ..contours2json import contours2json
from ..labels2contours import labels2contours


def get_edges(mask: np.ndarray):
    """get edges of the mask"""
    mask_copy = cv2.copyMakeBorder(
        mask,
        top=1,
        bottom=1,
        left=1,
        right=1,
        borderType=cv2.BORDER_CONSTANT,
        value=0,
    )
    edges = cv2.Canny(mask_copy, 50, 150)

    edges = cv2.dilate(edges, np.ones((5, 5)))

    edges = np.divide(edges, 255, dtype=np.float16)
    edges = edges.astype(np.uint8)
    edges = edges[1:-1, 1:-1]

    return edges


def mask2json(
    mask: np.ndarray,
    minimum_size_contours: int = 3,
    is_diagonal_connected=True,
    return_inner_contours: bool = False,
) -> list:
    """
    Converts binary mask to standard cytosmart format

    Args:
        mask (np.ndarray): binary mask
        minimum_size_contours (int, optional): The minimum number of points an contour should have to be included. Defaults to 3.
        is_diagonal_connected (bool, optional): If true diagonal pixels [[1, 0][0,1]] will be seen as the same object. Defaults to True.
        return_inner_contours (bool, optional): return the internal contours.
            These contours are around the holes with the contour
            default: False

    Returns:
        list: json_objects, 
            e.g [
                    {
                        'type':'polygon', 
                        'points':[
                            {'x':1,'y':4},
                            {'x':2,'y':3},
                            {'x':4,'y':4} 
                        ], 
                        "innerObjects": [
                            {
                                'type':'polygon', 
                                'points':[
                                    {'x':3,'y':2}
                                ]
                            }
                        ]
                    }
                ]
    """
    connectivity = 4
    if is_diagonal_connected:
        connectivity = 8

    labels = cv2.connectedComponents(mask.astype(np.uint8), connectivity=connectivity)[
        1
    ]
    edges = get_edges(mask)

    edges = edges * labels
    del labels
    contours = labels2contours(edges, return_inner_contours=return_inner_contours)

    if return_inner_contours:
        json_objects = []
        for contour in contours:
            json_object = contours2json([contour[0]])[0]
            json_object["innerObjects"] = contours2json(contour[1])
            json_objects.append(json_object)
    else:
        json_objects = contours2json(contours)
    json_objects = [
        json_object
        for i, json_object in enumerate(json_objects)
        if len(json_objects[i]["points"]) >= minimum_size_contours
    ]

    return json_objects



from typing import Union


def check_overlap_bbox(bb1: Union[tuple, list], bb2: Union[tuple, list]) -> bool:

    """
    Checks if two bounding boxes overlap with eachother.
    The bounding boxes are assumed the be structured as follows:
    (xmin, ymin, xmax, ymax).
    Warning: Touching bounding boxes are not seen as overlapping

    Args:
        bb1 (Union[tuple, list]): boundingbox 1, (x1, y1, x2 ,y2)
        bb2 (Union[tuple, list]): boundingbox 2, (x1, y1, x2 ,y2)

    Raises:
        ValueError: Length of bounding box is the wrong size

    Returns:
        bool: if the bounding boxes are touching
    """
    if len(bb1) != 4 or len(bb2) != 4:
        raise ValueError("The bounding boxes must be tuples or list of length 4")

    if (bb1[0] >= bb2[2]) or (bb2[0] >= bb1[2]):
        return False
    elif (bb1[1] >= bb2[3]) or (bb2[1] >= bb1[3]):
        return False
    else:
        return True


import copy


def flip_json(json_object: dict, img_dim: int) -> dict:
    """
    Takes in a json object (ellipse or polygon, standard CytoSMART format)
    and flips it over the y axis.
    json_object: (dict) following the standard CytoSMART format
    img_dim: (int) the y dimension of the image that is related to the json_object.

    input_json = {
            "type": "ellipse",
            "center": {"x": 9, "y": 3},
            "radiusX": 5,
            "radiusY": 5,
            "angleOfRotation": 0,
        }

    expected_json = {
        "type": "ellipse",
        "center": {"x": 9, "y": 6},
        "radiusX": 5,
        "radiusY": 5,
        "angleOfRotation": 0,
    }
    """
    newjson_object = copy.deepcopy(json_object)
    if json_object["type"] == "ellipse":
        newjson_object["center"]["y"] = img_dim - json_object["center"]["y"] - 1

    elif json_object["type"] == "polygon":
        for point_n in range(len(newjson_object["points"])):
            newjson_object["points"][point_n]["y"] = (
                img_dim - json_object["points"][point_n]["y"] - 1
            )
    else:
        raise TypeError(
            f"The type {json_object['type']} is not found in flipping augmentation"
        )

    return newjson_object



import numpy as np
import cv2

def relative_difference(img: np.ndarray, gauss_size: int = 301, smooth_size=None, do_normalize: bool= False, resize_ratio= None) -> np.array:
    """
    Remove big blurry artifact from the image.
    It will do this by taking the ratio difference between the blurred and raw image.
    
    # Usage
    This can be used for human vision.
    This is good as algorithm input for algorithm that rely on relative pixel difference.

    # NOT fractal
    This algorithm is NOT fractal. 
    Meaning the illumination correction followed by cropping will given a different result than first cropping.

    img (np.array): input image
    gauss_size (int): needs to be an odd number. 
        This is corralated to the size of the big artifact to remove.
    smooth_size (int): need to be an odd number.
        Is the size of the kernal of the gaussian filter that is used on the image before division.
    resize_ratio (float): the img is resized according to this ration before using a GaussianBlur.
        This is done to increase the speed of the function.
    do_normalize (bool): if true, the output img is normalized between the minimum and maximum
        value of the smoothed img. 
    """
    img = np.float32(img)
    
    if gauss_size % 2 == 0:
        gauss_size += 1

    if smooth_size:
        img = cv2.GaussianBlur(img, (smooth_size,smooth_size), 0)
    
    max_img = np.max(img)
    min_img = np.min(img)

    if resize_ratio:
        gb = cv2.resize(img, (0, 0), fx=resize_ratio, fy=resize_ratio, interpolation=cv2.INTER_LINEAR)
        gb = cv2.GaussianBlur(gb, (gauss_size, gauss_size), 0, borderType=cv2.BORDER_REPLICATE)
        gb = cv2.resize(gb, (img.shape[1], img.shape[0]), interpolation=cv2.INTER_LINEAR)
    else:
        gb = cv2.GaussianBlur(img, (gauss_size, gauss_size), 0)
        gb = np.float32(gb)

    img = np.divide(img, gb + 1e-15, dtype=np.float32)
    
    
    if do_normalize:
        img = cv2.normalize(img, img, max_img, min_img, cv2.NORM_MINMAX)
    else:
        img = np.divide(img, np.max(img) + 1e-15, dtype=np.float32) * 255
    
    return img.astype(np.uint8)


import numpy as np

from ... import img_dim
from ...make_mask import make_mask_contour, make_mask_ellipse
from ..json2contours import json2contours


def contour2mask(mask: np.ndarray, contour: list) -> np.ndarray:
    """Converts contour to mask"""
    add_mask = make_mask_contour(img_dim(mask), contour)
    mask = mask + add_mask
    mask[mask > 1] = 1
    return mask


def ellipse2mask(mask: np.ndarray, x: int, y: int, r1: float, r2: float) -> np.ndarray:
    """Converts ellipse to mask"""
    add_mask = make_mask_ellipse(img_dim(mask), x, y, r1, r2)
    mask = mask + add_mask
    mask[mask > 1] = 1
    return mask


def json2mask(
    json_objects: list, img_shape: tuple, minimum_size_contours: int = 3
) -> np.ndarray:
    """
    Convert standard cytosmart format to binary mask
    WARNING(1): objects with less than 3 components will get filtered out as default.
    Put minimum_size_contours to 0 if you want them included.
    WARNING(2): small objects will get some extra pixels when it has certain angles
    e.g json_object:

            {
                "type": "polygon",
                "points": [
                    {"x": 2, "y": 2},
                    {"x": 3, "y": 4},
                    {"x": 4, "y": 2},
                ],
            }

        expected result:

            [[0 0 0 0 0 0 0 0 0 0]
            [0 0 0 0 0 0 0 0 0 0]
            [0 0 1 1 1 0 0 0 0 0]
            [0 0 0 1 0 0 0 0 0 0]
            [0 0 0 1 0 0 0 0 0 0]
            [0 0 0 0 0 0 0 0 0 0]
            [0 0 0 0 0 0 0 0 0 0]
            [0 0 0 0 0 0 0 0 0 0]
            [0 0 0 0 0 0 0 0 0 0]
            [0 0 0 0 0 0 0 0 0 0]]

        result:

            [[0 0 0 0 0 0 0 0 0 0]
            [0 0 0 0 0 0 0 0 0 0]
            [0 0 1 1 1 0 0 0 0 0]
            [0 0 1 1 0 0 0 0 0 0]
            [0 0 0 1 0 0 0 0 0 0]
            [0 0 0 0 0 0 0 0 0 0]
            [0 0 0 0 0 0 0 0 0 0]
            [0 0 0 0 0 0 0 0 0 0]
            [0 0 0 0 0 0 0 0 0 0]
            [0 0 0 0 0 0 0 0 0 0]]

    This is a bug that should be removed in the future

    Args:
        json_objects (list): json with annotations in standard cytosmart format
        img_shape (tuple): the dimensions of the mask
        minimum_size_contours (int): The minimum number of points an contour should have to be included. Defaults to 3.

    Returns:
        np.ndarray: binary mask
    """
    mask = np.zeros(img_shape, dtype=np.uint8)
    for json_object in json_objects:
        if json_object["type"] == "polygon":
            if len(json_object["points"]) >= minimum_size_contours:

                contour = json2contours(json_object)
                contour = [x[0] for x in contour]
                mask = contour2mask(mask, contour)
        elif json_object["type"] == "ellipse":
            x = json_object["center"]["x"]
            y = json_object["center"]["y"]
            r1 = json_object["radiusX"]
            r2 = json_object["radiusY"]
            mask = ellipse2mask(mask, x, y, r1, r2)
        else:
            raise TypeError(
                f"json object {json_object['type']} could not be converted to a mask"
            )

    return mask



from __future__ import absolute_import
from unittest import TestCase
import numpy as np
from .main import absolute_difference


class TestIlluminationCorretionAlgorithm(TestCase):
    def test_grey_image(self):
        img = np.ones((1000, 1000)) * 128
        illum_img = absolute_difference(img)
        self.assertGreaterEqual(np.min(illum_img), 0)
        self.assertLessEqual(np.max(illum_img), 255)

    def test_white_image(self):
        img = np.ones((1000, 1000)) * 255
        illum_img = absolute_difference(img)
        self.assertGreaterEqual(np.min(illum_img), 0)
        self.assertLessEqual(np.max(illum_img), 255)

    def test_black_image(self):
        img = np.ones((1000, 1000)) * 0
        illum_img = absolute_difference(img)
        self.assertGreaterEqual(np.min(illum_img), 0)
        self.assertLessEqual(np.max(illum_img), 255)

    def test_illumination_black_white(self):
        # BLACK GREY WHITE TEST
        xx, yy = np.mgrid[:300, :300]
        dist = xx
        img = (dist >= 100) * 128 + (dist >= 200) * 128
        illum_img = absolute_difference(img, gauss_size=21)
        self.assertGreaterEqual(np.min(illum_img), 0)
        self.assertLessEqual(np.max(illum_img), 255)

    def test_illumination_3_shades_of_grey(self):
        print("\nSTART 3 SHADES OF GREY TEST")
        xx, yy = np.mgrid[:300, :300]
        dist = xx
        img = (dist >= 100) * 50 + (dist >= 200) * 50 + 50
        illum_img = absolute_difference(img, gauss_size=21)
        self.assertGreaterEqual(np.min(illum_img), 0)
        self.assertLessEqual(np.max(illum_img), 255)

    def test_fractal(self):
        np.random.seed(931224)
        img = np.random.randint(0, 255, (1000, 1000))
        img[0:400, 0:400] = np.zeros((400, 400))
        cent_img = img[400:600, 400:600]

        full_illum_img = absolute_difference(img, gauss_size=21, new_median=128)
        small_illum_img = absolute_difference(cent_img, gauss_size=21, new_median=128)

        np.testing.assert_array_equal(
            small_illum_img[50:150, 50:150], full_illum_img[450:550, 450:550]
        )

    def test_warning(self):
        # BLACK GREY WHITE TEST
        xx, yy = np.mgrid[:300, :300]
        dist = xx
        img = (dist >= 100) * 128 + (dist >= 200) * 128

        with self.assertWarns(SyntaxWarning):
            illum_img = absolute_difference(img, gauss_size=20)

        self.assertGreaterEqual(np.min(illum_img), 0)
        self.assertLessEqual(np.max(illum_img), 255)


from unittest import TestCase
from .main import add_circularity
import math


class TestScaleObject(TestCase):
    def test_square(self):
        input_json_object = {
            "type": "polygon",
            "points": [
                {"x": 5000, "y": 0},
                {"x": 5000, "y": 10000},
                {"x": 15000, "y": 10000},
                {"x": 15000, "y": 0},
            ],
            "id": "unicorn",
        }

        expected_json_object = {
            "type": "polygon",
            "points": [
                {"x": 5000, "y": 0},
                {"x": 5000, "y": 10000},
                {"x": 15000, "y": 10000},
                {"x": 15000, "y": 0},
            ],
            "id": "unicorn",
            "circularity": 2 / math.pi,
        }

        add_circularity(input_json_object)
        self.assertAlmostEqual(
            input_json_object["circularity"], expected_json_object["circularity"], 5
        )

        # check everything except circularity
        input_json_object["circularity"] = 0
        expected_json_object["circularity"] = 0
        self.assertDictEqual(input_json_object, expected_json_object)

    def test_unknown_type(self):
        input_json_object = {
            "type": "unicornType",
        }
        with self.assertRaises(ValueError):
            add_circularity(input_json_object)



from unittest import TestCase
import numpy as np 
from .main import img_dim

class Test_img_dim(TestCase):
    def test_happyflow_no_channel(self):
        inArr = np.zeros((10, 20))
        expected = [20, 10]
        result = img_dim(inArr, False)

        np.testing.assert_array_equal(result, expected)

    def test_happyflow_no_given_channel(self):
        inArr = np.zeros((10, 20))
        expected = [20, 10, 1]
        result = img_dim(inArr, True)

        np.testing.assert_array_equal(result, expected)

    def test_happyflow_with_channel(self):
        inArr = np.zeros((10, 20, 3))
        expected = [20, 10, 3]
        result = img_dim(inArr, True)

        np.testing.assert_array_equal(result, expected)

    def test_happyflow_with_noshow_channel(self):
        inArr = np.zeros((10, 20, 3))
        expected = [20, 10]
        result = img_dim(inArr, False)

        np.testing.assert_array_equal(result, expected)


import numpy as np
from typing import List
from ..ellipse2json import ellipse2json
from ..list_of_points2json import list_of_points2json


def vgg2json(vgg_data: dict) -> List[dict]:
    """
    Transforms a created with 'VGG Image Annotator' into a lists of jsons
    vgg_data: (dict) a opened json made with the VGG. Can contain annotations of
        multiple images.
    """
    results = []

    for filename in vgg_data.keys():

        result = []

        for annotation_vgg in vgg_data[filename]["regions"]:
            json_object = {}
            vgg_type = annotation_vgg["shape_attributes"]["name"]
            if vgg_type == "polygon":
                x_list, y_list = (
                    annotation_vgg["shape_attributes"]["all_points_x"],
                    annotation_vgg["shape_attributes"]["all_points_y"],
                )
                list_of_points = np.array([x_list, y_list]).transpose()
                json_object = list_of_points2json(list_of_points)
            elif vgg_type == "circle":
                cx, cy = (
                    annotation_vgg["shape_attributes"]["cx"],
                    annotation_vgg["shape_attributes"]["cy"],
                )
                r = annotation_vgg["shape_attributes"]["r"]
                json_object = ellipse2json(cx, cy, r)
            elif vgg_type == "ellipse":
                cx, cy = (
                    annotation_vgg["shape_attributes"]["cx"],
                    annotation_vgg["shape_attributes"]["cy"],
                )
                rx, ry = (
                    annotation_vgg["shape_attributes"]["rx"],
                    annotation_vgg["shape_attributes"]["ry"],
                )
                alpha = annotation_vgg["shape_attributes"]["theta"]
                json_object = ellipse2json(cx, cy, rx, ry, alpha)
            result.append(json_object)
        results.append(result)

    return results



from __future__ import absolute_import
from unittest import TestCase
from .main import json2vgg


class TestJson2VGG(TestCase):
    def test_select_vgg_2_Json_one_file(self):
        input_json_list = [
            {
                "type": "polygon",
                "accuracy": 0.45455,
                "parents": [],
                "children": [],
                "id": "unicorn",
                "points": [
                    {"x": 505, "y": 745},
                    {"x": 478, "y": 756},
                ],
            },
            {
                "type": "ellipse",
                "accuracy": 0.443,
                "id": "unicorn",
                "center": {"x": 769, "y": 649},
                "radiusX": 24,
                "radiusY": 23,
                "angleOfRotation": 0,
            },
        ]

        expected_vgg_json_output = {
            "test1.jpg": {
                "filename": "test1.jpg",
                "regions": [
                    {
                        "shape_attributes": {
                            "name": "polygon",
                            "all_points_x": [
                                505,
                                478,
                            ],
                            "all_points_y": [
                                745,
                                756,
                            ],
                        },
                        "region_attributes": {"accuracy": 0.45455},
                    },
                    {
                        "shape_attributes": {
                            "name": "ellipse",
                            "cx": 769,
                            "cy": 649,
                            "rx": 24,
                            "ry": 23,
                            "theta": 0,
                        },
                        "region_attributes": {"accuracy": 0.443},
                    },
                ],
                "file_attributes": {},
            },
        }

        result = json2vgg(input_json_list, "test1")

        for i in range(len(result)):
            self.assertDictEqual(result, expected_vgg_json_output)

    def test_select_vgg_2_Json_one_file_no_accuracy(self):
        input_json_list = [
            {
                "type": "polygon",
                "accuracy": 0.45455,
                "parents": [],
                "children": [],
                "id": "unicorn",
                "points": [
                    {"x": 505, "y": 745},
                    {"x": 478, "y": 756},
                ],
            },
            {
                "type": "ellipse",
                "accuracy": 0.443,
                "id": "unicorn",
                "center": {"x": 769, "y": 649},
                "radiusX": 24,
                "radiusY": 23,
                "angleOfRotation": 0,
            },
        ]

        expected_vgg_json_output = {
            "test1.jpg": {
                "filename": "test1.jpg",
                "regions": [
                    {
                        "shape_attributes": {
                            "name": "polygon",
                            "all_points_x": [
                                505,
                                478,
                            ],
                            "all_points_y": [
                                745,
                                756,
                            ],
                        },
                        "region_attributes": {},
                    },
                    {
                        "shape_attributes": {
                            "name": "ellipse",
                            "cx": 769,
                            "cy": 649,
                            "rx": 24,
                            "ry": 23,
                            "theta": 0,
                        },
                        "region_attributes": {},
                    },
                ],
                "file_attributes": {},
            },
        }

        result = json2vgg(input_json_list, "test1", add_accuracy=False)

        for i in range(len(result)):
            self.assertDictEqual(result, expected_vgg_json_output)
